/* ALMALLOC.C  -  Aligned memory allocation
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * March 13, 2003
 */

#include <atari.h>
#include <stdlib.h>
#include <stdio.h>


extern void __fastcall__ _heapadd(void *mem, size_t size);



/* aligned_malloc
 * 
 * Allocates an aligned block of memory of size, aligned to boundary
 * that you pass in.  Returns pointer to the memory block or NULL if
 * error allocating memory.
 */
void __fastcall__ *aligned_malloc(size_t size, unsigned int bound)
{
  char *d, *b;                          //           |
  unsigned int lo_free;                 //           |
  unsigned int hi_free;                 //           | 
  unsigned int bnd;


  bnd = bound - 1;


  //*** Allocate an aligned block of memory ***//
  d = (char *) malloc(size + bnd);                          // allocate mem
  if (!d) return(NULL);                                     // mem not avail

  b = (char *) ((((unsigned int) d) + bnd) & ~bnd);         // aligned pointer

  lo_free = b - d;                                          // memory not used
  hi_free = bnd - lo_free;                                  //       |
  _heapadd(d, lo_free);                                         // free low
  _heapadd(b + bound, hi_free);                                 // free high

  return(b);
}
