/* ATARIGFX.H  -  Atari Graphics Library
 *
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#ifndef ATARIGFX_H
#define ATARIGFX_H

#include <stdlib.h>


/********************* Hardware address defines ***********************/
//*** Shadows ***//
#define SDMCTL  *(unsigned char *)  0x022F          // dma control shadow
#define SDLSTL  *(unsigned int *)   0x0230          // DL address shadow
#define STICK0  *(unsigned char *)  0x0278          // joystick0 direction
#define STICK1  *(unsigned char *)  0x0279          // joystick1 direction
#define STICK2  *(unsigned char *)  0x0280          // joystick2 direction
#define STICK3  *(unsigned char *)  0x0281          // joystick3 direction
#define STRIG0  *(unsigned char *)  0x0284          // joystick0 button 
#define STRIG1  *(unsigned char *)  0x0285          // joystick1 button 
#define STRIG2  *(unsigned char *)  0x0286          // joystick2 button 
#define STRIG3  *(unsigned char *)  0x0287          // joystick3 button 
#define CHBAS   *(unsigned char *)  0x02F4          // charset address shadow
#define PCOLR0  *(unsigned char *)  0x02C0          // PM0 color
#define PCOLR1  *(unsigned char *)  0x02C1          // PM1 color
#define PCOLR2  *(unsigned char *)  0x02C2          // PM2 color
#define PCOLR3  *(unsigned char *)  0x02C3          // PM3 color
#define COLOR0  *(unsigned char *)  0x02C4          // PF0 color
#define COLOR1  *(unsigned char *)  0x02C5          // PF1 color
#define COLOR2  *(unsigned char *)  0x02C6          // PF2 color
#define COLOR3  *(unsigned char *)  0x02C7          // PF3 color
#define COLOR4  *(unsigned char *)  0x02C8          // PF4 color
#define GPRIOR  *(unsigned char *)  0x026F          // graphics priority reg
//*** GTIA Registers ***//
#define HPOSP0  *(unsigned char *)  0xD000          // (w) player 0 horiz pos.
#define HPOSP1  *(unsigned char *)  0xD001          // (w) player 1 horiz pos.
#define HPOSP2  *(unsigned char *)  0xD002          // (w) player 2 horiz pos.
#define HPOSP3  *(unsigned char *)  0xD003          // (w) player 3 horiz pos.
#define HPOSM0  *(unsigned char *)  0xD004          // (w) missile 0 horiz pos.
#define HPOSM1  *(unsigned char *)  0xD005          // (w) missile 0 horiz pos.
#define HPOSM2  *(unsigned char *)  0xD006          // (w) missile 0 horiz pos.
#define HPOSM3  *(unsigned char *)  0xD007          // (w) missile 0 horiz pos.
#define SIZEP0  *(unsigned char *)  0xD008          // (w) player 0 size
#define SIZEP1  *(unsigned char *)  0xD009          // (w) player 1 size
#define SIZEP2  *(unsigned char *)  0xD00A          // (w) player 2 size
#define SIZEP3  *(unsigned char *)  0xD00B          // (w) player 3 size
#define SIZEM   *(unsigned char *)  0xD00C          // (w) missile size
#define GRAFP0  *(unsigned char *)  0xD00D          // (w) P0 gfx reg.
#define GRAFP1  *(unsigned char *)  0xD00E          // (w) P1 gfx reg.
#define GRAFP2  *(unsigned char *)  0xD00F          // (w) P2 gfx reg.
#define GRAFP3  *(unsigned char *)  0xD010          // (w) P3 gfx reg.
#define GRAFM   *(unsigned char *)  0xD011          // (w) missile gfx reg.
#define COLPM0  *(unsigned char *)  0xD012          // PM0 color reg
#define COLPM1  *(unsigned char *)  0xD013          // PM1 color reg
#define COLPM2  *(unsigned char *)  0xD014          // PM2 color reg
#define COLPM3  *(unsigned char *)  0xD015          // PM3 color reg
#define COLPF0  *(unsigned char *)  0xD016          // Playfield 0 color reg
#define COLPF1  *(unsigned char *)  0xD017          // Playfield 1 color reg
#define COLPF2  *(unsigned char *)  0xD018          // Playfield 2 color reg
#define COLPF3  *(unsigned char *)  0xD019          // Playfield 3 color reg
#define COLBK   *(unsigned char *)  0xD01A          // Playfield backgrnd reg
#define PRIOR   *(unsigned char *)  0xD01B          // Graphics priority reg.
#define VDELAY  *(unsigned char *)  0xD01C          // (w) vertical delay reg
#define GRACTL  *(unsigned char *)  0xD01D          // (w) GTIA gfx control
#define HITCLR  *(unsigned char *)  0xD01E          // (w) collision clear reg
#define CONSOL  *(unsigned char *)  0xD01F          // Console keys reg
#define M0PF    *(unsigned char *)  0xD000          // (r) M0 to PF col reg.
#define M1PF    *(unsigned char *)  0xD001          // (r) M1 to PF col reg.
#define M2PF    *(unsigned char *)  0xD002          // (r) M2 to PF col reg.
#define M3PF    *(unsigned char *)  0xD003          // (r) M3 to PF col reg.
#define P0PF    *(unsigned char *)  0xD004          // (r) P0 to PF col reg.
#define P1PF    *(unsigned char *)  0xD005          // (r) P1 to PF col reg.
#define P2PF    *(unsigned char *)  0xD006          // (r) P2 to PF col reg.
#define P3PF    *(unsigned char *)  0xD007          // (r) P3 to PF col reg.
#define M0PL    *(unsigned char *)  0xD008          // (r) M0 to player col reg
#define M1PL    *(unsigned char *)  0xD009          // (r) M1 to player col reg
#define M2PL    *(unsigned char *)  0xD00A          // (r) M2 to player col reg
#define M3PL    *(unsigned char *)  0xD00B          // (r) M3 to player col reg
#define P0PL    *(unsigned char *)  0xD00C          // (r) P0 to player col reg
#define P1PL    *(unsigned char *)  0xD00D          // (r) P1 to player col reg
#define P2PL    *(unsigned char *)  0xD00E          // (r) P2 to player col reg
#define P3PL    *(unsigned char *)  0xD00F          // (r) P3 to player col reg
#define TRIG0   *(unsigned char *)  0xD010          // (r) stick0 trigger
#define TRIG1   *(unsigned char *)  0xD011          // (r) stick1 trigger
#define TRIG2   *(unsigned char *)  0xD012          // (r) stick2 trigger
#define TRIG3   *(unsigned char *)  0xD013          // (r) stick3 trigger
#define PAL     *(unsigned char *)  0xD014          // (r) stick3 trigger

#define PMBASE  *(unsigned char *)  0xD407          // player missile base addr


/*************************** Data structures **************************/


/* Bitmap structure
 *
 * To keep track of bitmaps in memory for drawing, etc...
 */
typedef struct BITMAP {
	   char *dat;               // pointer to actual data
  unsigned char mode;               // graphics mode of bitmap (antic)
  unsigned char bpl;                // bytes per line (max = 255)
  //unsigned char clip;               // clipping?
   unsigned int w, h;               // width and height of bitmap
} BITMAP;

/* Display List setup structure
 *
 *  mode, lines, bitmap, hscroll, vscroll, dli, dli_address
 *
 * Last entry should be a FF for the mode line.
 */
typedef struct DL_SETUP_T {
  unsigned char mode;           // Antic mode
  unsigned char lines;          // number of antic mode lines
  BITMAP        *bmp;           // Bitmap pointer
  unsigned char hscroll;        // Horizontal scroll enable?
  unsigned char vscroll;        // Vertical scroll enable?
  unsigned char dli_flag;       // DLI flag
  void          *dli_address;   // DLI address 
} DL_SETUP_T;

/* Display list structure
 */
typedef struct DLIST {
  DL_SETUP_T *setup;
  char *dat;
} DLIST;

/* Player/Missile Area structure
 *
 * To keep track of player missile areas, resolution, etc...
 */
typedef struct PMAREA {
  unsigned char *dat;               // pointer to data
  unsigned char mode;               // single/double line resolution (1k/2k)
} PMAREA; 

/* Sprite structure
 *
 * These sprites are generally within the PMAREA.
 */
typedef struct SPRITE {
  unsigned char *dat;                   // pointer to data (inside pmarea usually)
  //unsigned int x, y;                    // position (x not used for pm sprites)
  unsigned int w, h;                    // width and height (w=8 for pm sprites) 
} SPRITE;

/******************************* Helpers ******************************/

//extern const char _bpl[];
extern const char _ppb[];
extern const char _bak[];

#define BYTES_PER_LINE(bmp) (bmp->w / PIXELS_PER_BYTE(bmp->mode))
#define PIXELS_PER_BYTE(x)  _ppb[x-2]
#define BAKCOL(x)           _bak[x-2]

extern const char plot2cmask[8];
extern const char plot4cmask[4];
extern const char plot4cshift[4];

/*************************** Display list *****************************/

char __fastcall__ *create_display_list(DL_SETUP_T *list);
void __fastcall__ destroy_display_list(char *b);
void __fastcall__ activate_display_list(char *dl);


/***************************** Bitmaps *******************************/

BITMAP __fastcall__ *create_bitmap(unsigned char mode, unsigned int w, unsigned int h);
BITMAP __fastcall__ *destroy_bitmap(BITMAP *bmp);
void __fastcall__ clear_bitmap_to(BITMAP *bmp, unsigned char creg);
void __fastcall__ clear_bitmap(BITMAP *bmp);
unsigned char __fastcall__ load_bitmap_pic(BITMAP *bmp, char *filename);
unsigned char __fastcall__ load_bitmap_mic(BITMAP *bmp, char *filename);

/******************************* Drawing *****************************/

void __fastcall__ plot_4colmode_nc(BITMAP *bmp, unsigned int x, unsigned int y, unsigned char creg);
void __fastcall__ plot_4colmode(BITMAP *bmp, int x, int y, unsigned char creg);
void __fastcall__ plot_2colmode_nc(BITMAP *bmp, unsigned int x, unsigned int y, unsigned char creg);
void __fastcall__ plot_2colmode(BITMAP *bmp, int x, int y, unsigned char creg);
void __fastcall__ plot_gtiamode_nc(BITMAP *bmp, unsigned int x, unsigned int y, unsigned char creg);
void __fastcall__ plot_gtiamode(BITMAP *bmp, int x, int y, unsigned char creg);

void __fastcall__ rectfill_4colmode_nc(BITMAP *bmp, unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char creg);

/****************************** Blitting *****************************/

void __fastcall__ blit_aligned_nc(BITMAP *src, BITMAP *dest, int sx, int sy, int dx, int dy, int w, int h);
void __fastcall__ blit_nc(BITMAP *src, BITMAP *dest, int sx, int sy, int dx, int dy, int w, int h);
void __fastcall__ blit(BITMAP *src, BITMAP *dest, int sx, int sy, int dx, int dy, int w, int h);


/******************************* Charset *****************************/

#define FONT        unsigned char
#define sysfont     ((FONT *) 0xE000)        // main system font
#define intfont     ((FONT *) 0xCC00)        // international font (XL/XE)

void __fastcall__ set_system_font(FONT *font);
FONT __fastcall__ *font_copy(FONT *font);
FONT __fastcall__ *load_font(char *filename);
unsigned char __fastcall__ save_font(FONT *font, char *filename);

//FONT __fastcall__ *create_font();
void __fastcall__ destroy_font(FONT *font);


/******************************* Textout *****************************/

char __fastcall__ atascii_to_internal(char c);
void __fastcall__ str_to_internal(char *s);

void __fastcall__ textout_textmode_nc(BITMAP *bmp, const char *s, int x, int y);
void __fastcall__ textout_textmode(BITMAP *bmp, const char *s, int x, int y);
void __fastcall__ textout_2colmode_aligned_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y);
void __fastcall__ textout_2colmode_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);
void __fastcall__ textout_2colmode(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);
void __fastcall__ textout_4colmode_nc(BITMAP *bmp, FONT *font, const char *s, unsigned int x, unsigned int y, unsigned char creg);
void __fastcall__ textout_4colmode(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);
void __fastcall__ textout_gtiamode_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);
void __fastcall__ textout_gtiamode(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);

void __fastcall__ textout_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);
void __fastcall__ textout(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg);


/******************************* PM Area *****************************/

#define     PM_1LINE        0               // single line resolution
#define     PM_2LINE        1               // double line resolution

#define     PM_1L_MOFF      768             // offset to missiles, 1 line res.
#define     PM_1L_P0OFF     1024            // offset to player0
#define     PM_1L_P1OFF     1280            // offset to player1
#define     PM_1L_P2OFF     1536            // offset to player2
#define     PM_1L_P3OFF     1792            // offset to player3
#define     PM_2L_MOFF      384             // offset to missiles, 1 line res.
#define     PM_2L_P0OFF     512             // offset to player0
#define     PM_2L_P1OFF     640             // offset to player1
#define     PM_2L_P2OFF     768             // offset to player2
#define     PM_2L_P3OFF     896             // offset to player3

PMAREA __fastcall__ *create_pmarea(unsigned char mode);
void __fastcall__ destroy_pmarea(PMAREA *pmarea);
void init_pmgraphics(void);
void __fastcall__ set_active_pmarea(PMAREA *pmarea);


/******************************* Sprites *****************************/

SPRITE __fastcall__ *create_sprite(BITMAP *bmp);
SPRITE __fastcall__ *create_pmsprite(BITMAP *bmp);
void __fastcall__ destroy_sprite(SPRITE *sprite);

void __fastcall__ draw_sprite(BITMAP *bmp, SPRITE *sprite, int x, int y);
void __fastcall__ erase_sprite(BITMAP *bmp, SPRITE *sprite, int x, int y);


/* Helpers */

void __fastcall__ *aligned_malloc(size_t size, unsigned int bound);


#endif
