/* BLITSMC.C  - Blit fucntion, non-byte aligned, no conversion, does clipping
 *
 * Atari Graphics Library
 * Shawn Jefferson
 * 
 * December 18, 2002
 */

#include <atari.h>
#include <conio.h>
#include <string.h>
//#include <dbg.h>
#include "atarigfx.h"



/* blit
 *
 * Blit from the source bitmap to the destination bitmap.  This function
 * assumes that the graphics modes of the bitmap is the same. Source and
 * destination locations do not need to be byte aligned.  We are copying
 * pixel by pixel. This function does clipping at the destination bitmap.
 */
void __fastcall__ blit(BITMAP *src, BITMAP *dest, int sx, int sy, int dx, int dy, int w, int h)
{
  unsigned char sbx;
  unsigned char spx;               // source byte and pixel counter
  unsigned char dbx;
  unsigned char dpx;               // dest byte and pixel counter
  unsigned int spos;                    // source bitmap position
  unsigned int dpos;                    // destination bitmap position
  unsigned char pixel;                  // pixel
  unsigned char mask;                   // mask
  unsigned int i;                       // line counter
  unsigned int j;                       // pixel counter


  //*** Initialize ***//
  sbx = sx / PIXELS_PER_BYTE(src->mode);            // source byte
  spx = sx % PIXELS_PER_BYTE(src->mode);            // source pixel
  dbx = dx / PIXELS_PER_BYTE(dest->mode);           // dest byte
  dpx = dx % PIXELS_PER_BYTE(dest->mode);           // dest pixel

  spos = (sy * BYTES_PER_LINE(src)) + sbx;          // source pos. in bitmap
  dpos = (dy * BYTES_PER_LINE(dest)) + dbx;         // dest pos. in bitmap

  sbx = (w / PIXELS_PER_BYTE(src->mode));           // width in bytes
  dbx = (w / PIXELS_PER_BYTE(dest->mode));          // width in bytes (same?)

  //*** Copy each line ***//
  for(i=0; i<h; i++) {                              // height loop
    /* Clip height */
    if ((dy + i) > dest->h)                         // outside dest bitmap?
      break;                                        // exit loop

    for(j=0; j<w; j++) {                            // width loop
      /* Clip width */
      if ((dx + j) > dest->w)                       // outside dest bitmap?
	break;                                      // exit loop

      pixel = src->dat[spos];                       // grab source pixel

      switch (PIXELS_PER_BYTE(src->mode)) {         // seperate it from byte
	case 8:
	  mask = 0x01;                              // pixel mask
	  mask = mask << (7 - spx);                 //     |

	  pixel &= mask;                            // mask it out
	  pixel = pixel >> (7 - spx);               // shift it down
	  break;

	case 4:
	  mask = 0x03;
	  mask = mask << ((3 - spx) * 2);

	  pixel &= mask;
	  pixel = pixel >> ((3 - spx) * 2); 
	  break;

	case 2:
	  mask = 0x0F;
	  mask = mask << ((1 - spx) * 4);

	  pixel &= mask;
	  pixel = pixel >> ((1 - spx) * 4); 
	  break;
      }

      switch (PIXELS_PER_BYTE(dest->mode)) {        // draw in destination
	case 8:
	  mask = 0x01;                              // dest pixel mask
	  mask = mask << (7 - dpx);                 //       | 

	  dest->dat[dpos] &= ~mask;                 // clear the pixel

	  pixel = pixel << (7 - dpx);               // shift src pixel value
	  dest->dat[dpos] |= pixel;                 // draw in the dest byte
	  break;

	case 4:
	  mask = 0x03;
	  mask = mask << ((3 - dpx) * 2);

	  dest->dat[dpos] &= ~mask;

	  pixel = pixel << ((3 - dpx) * 2);
	  dest->dat[dpos] |= pixel;
	  break;

	case 2:
	  mask = 0x0F;
	  mask = mask << ((1 - dpx) * 4);

	  dest->dat[dpos] &= ~mask;

	  pixel = pixel << ((1 - dpx) * 4);
	  dest->dat[dpos] |= pixel;
	  break;
      }

      spx++;                                        // next source pixel
      if (spx == PIXELS_PER_BYTE(src->mode)) {      // next byte?
	spos++; 
	spx = 0;
      }

      dpx++;                                        // next dest pixel
      if (dpx == PIXELS_PER_BYTE(dest->mode)) {     // next byte?
	dpos++;
	dpx = 0;
      }
    }

    spos += BYTES_PER_LINE(src) - sbx;              // next line in src bitmap
    dpos += BYTES_PER_LINE(dest) - dbx;             // next line in dest bitmap
  }

  return;
}
