/* BMPCREAT.C  -  Create bitmap
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <stdio.h>
#include "atarigfx.h"


extern void __fastcall__ _heapadd(void *mem, size_t size);



/* create_bitmap
 *
 * Creates a bitmap for drawing or printing text to.  Pass in the antic
 * mode of this bitmap and the height and width.  Returns a pointer to
 * a BITMAP structure, or NULL if some error.  Free the memory with the
 * destroy_bitmap function.
 */ 
BITMAP __fastcall__ *create_bitmap(unsigned char mode, unsigned int w, unsigned int h)
{
  char *b, *d;
  unsigned int lo_free;
  unsigned int hi_free;
  unsigned int bytes;
  unsigned char extra;
  BITMAP *bmp;


  //*** Allocate memory for bitmap structure ***//
  bmp = (BITMAP *) malloc(sizeof(BITMAP));
  if (!bmp) return(NULL);

  bmp->mode = mode;
  bmp->w    = w;
  bmp->h    = h;
  bmp->bpl  = BYTES_PER_LINE(bmp);
  //bmp->clip = 1;

  bytes = (bmp->bpl * h);                           // total bytes for buffer
  if (bytes > 4096) {
    extra = (unsigned char) (4096 / bmp->bpl);      // # of screen lines fit
    extra = (unsigned char) (4096 - (bmp->bpl * extra)); 
  }
  else
    extra = 0; 

  //*** Align on a 4k boundary if bitmap less than 4k ***// 
  d = (char *) malloc((bytes) + 0xFFF + extra);          // allocate mem
  b = (char *) ((((unsigned int) d) + 0xFFF) & ~0xFFF);  // aligned pointer
  b = b + extra;                                         // add extra

  lo_free = b - d;                                       // memory not used
  hi_free = 0xFFF - lo_free;                             //       |
  _heapadd(d, lo_free);                                      // free low chunk
  _heapadd(b + (bytes + extra), hi_free);                    // free high chunk

  bmp->dat = b;
  if (!bmp->dat) {
    free(bmp);
    return(NULL);
  }

  return(bmp);
}
