/* DLCREATE.C  -  Create display list
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"



//*** Antic mode instruction bits ***//
#define ANTIC_DLI   128
#define ANTIC_LMS   64
#define ANTIC_VSCRL 32
#define ANTIC_HSCRL 16

#define MAX_DL_LEN      1024



/* create_display_list
 *
 * Creates the display list from the display list structure, allocates
 * memory for it (on a 1k boundary) and returns the address to the user.
 * The display list is not active until the user activates it.
 */
char __fastcall__ *create_display_list(DL_SETUP_T *list)
{
  char *dl;                             // temporary DL storage
  unsigned int *t;                      // aligned memory pointers
  char *b;                              //           |
  unsigned int lo_free;                 //           |
  unsigned int hi_free;                 //           | 
  int m;                                // dl mode instruction
  int i, n, l;                          // counters
  unsigned int bytes;                   // bytes to line


  //*** Initialize ***//
  i = 0;                                // location in temp array
  n = 0;                                // display list setup entry
  dl = (char *) malloc(MAX_DL_LEN);     // allocate temp storage for dl
  if (!dl) return(NULL);

  //*** Build display list ***//
  do {
    if (list[n].mode == 0xFF) break;                      // last setup entry

    if ((list[n].mode < 0x10) && (list[n].mode > 1)) {    // antic mode entry
      //*** Deal with first line ***//
      m = list[n].mode;

      if (list[n].hscroll)  m += ANTIC_HSCRL;             // horiz fine scroll
      if (list[n].vscroll)  m += ANTIC_VSCRL;             // vert fine scroll

      if ((list[n].dli_flag) && (list[n].lines == 1))
	 m += ANTIC_DLI;                                  // DLI flag 

      if (list[n].bmp) {                                  // Load memory scan
	m += ANTIC_LMS;
	dl[i] = m;
	i++; 

	t = (unsigned int *) &dl[i];                      // LMS address
	*t = (unsigned int) list[n].bmp->dat;             //     | 
	i += 2;
      }

      //*** Deal with rest of the lines ***//
      for(l=1; l < list[n].lines; l++) {
	m = list[n].mode;                                 // mode

	if ((list[n].dli_flag) && (l == list[n].lines-1)) // dli on last line
	  m += ANTIC_DLI;

	if (list[n].vscroll)  m += ANTIC_VSCRL;           // vert fine scroll

	bytes = (unsigned int) (l * (list[n].bmp->bpl));

	if (list[n].hscroll) {                            // horiz fine scroll
	  m += ANTIC_HSCRL;
	  m += ANTIC_LMS;
	  dl[i] = m;
	  i++; 

	  t = (unsigned int *) &dl[i];                      // LMS address
	  *t = (unsigned int) list[n].bmp->dat + bytes;
	  i += 2;

	  continue;
	}

	//*** 4k boundary LMS instruction? ***//
	//lo_free = (unsigned int) (4096 / (list[n].bmp->bpl);

	if (l == 102) {
	  m += ANTIC_LMS;
	  dl[i] = m;
	  i++; 

	  t = (unsigned int *) &dl[i];                      // LMS address
	  *t = (unsigned int) (list[n].bmp->dat + bytes);   //     |
	  i += 2;

	  continue;
	}

	dl[i] = m;
	i++;
      }
    }
    else {
      dl[i] = list[n].mode;             // blank line statements
      i++;
    }

    n++;                                // next line in dl setup list
  } while (1);

  //*** Allocate 1k aligned block for display list ***//
  b = aligned_malloc(i+4, 1024);            // 1k aligned block

  //*** Jump and wait for vblank instruction ***//
  dl[i] = 65;
  i++;
  t = (unsigned int *) &dl[i];              // JVB address (start of list)
  *t = (unsigned int) b;                    //        |
  i += 2;                                   // dl array index

  memcpy(b, dl, i);                         // copy the dl into allocd mem
  free(dl);                                 // free our temporary array
  return(b);
}
