/* FONTCOPY.C  -  Copy an existing font to a new one
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * Feb. 06, 2003
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"


extern void __fastcall__ _heapadd(void *mem, size_t size);



/* font_copy
 *
 * Copy the contents of one font into another one.  Allocates memory for
 * the destination font on the heap.  Call destroy_font to free the memory.
 * returns a pointer to the new font, or NULL if some error occurred.
 */
FONT __fastcall__ *font_copy(FONT *font)
{
  FONT *b, *d;
  unsigned int lo_free;
  unsigned int hi_free;


  //*** Allocate memory for new font ***//
  //*** Align on a 1k boundary ***// 
  d = (char *) malloc((sizeof(char)*1024) + 0x3FF);       // allocate mem
  b = (char *) ((((unsigned int) d) + 0x3FF) & ~0x3FF);  // aligned pointer

  lo_free = b - d;                                       // memory not used
  hi_free = 0x3FF - lo_free;                             //       |
  _heapadd(d, lo_free);                                      // free low chunk
  _heapadd(b + 0x3FF, hi_free);                              // free high chunk

  memcpy(b, font, 1024);                              // copy the font

  return(b);
}