/* FNTLOAD.C  -  Font load function
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * Feb 6, 2003
 */

#include <atari.h>
#include <stdlib.h>
#include <stdio.h>
#include "atarigfx.h"


extern void __fastcall__ _heapadd(void *mem, size_t size);



/* load_font
 *
 * Load a font into memory and return a pointer to it.  Call destroy font
 * to free the memory.  Returns NULL if error occurred.
 */
FONT __fastcall__ *load_font(char *filename)
{
  FILE *fp;
  FONT *b, *d;
  unsigned int lo_free;
  unsigned int hi_free;
  long n;


  //*** Open the file ***//
  fp = fopen(filename, "rb");
  if (!fp) return(NULL);

  //*** Allocate memory for new font ***//
  //*** Align on a 1k boundary ***// 
  d = (char *) malloc((sizeof(char)*1024) + 0x3FF);       // allocate mem
  if (!d) {                                               // no mem available 
    fclose(fp);
    return(NULL);
  }

  b = (char *) ((((unsigned int) d) + 0x3FF) & ~0x3FF);  // aligned pointer

  lo_free = b - d;                                       // memory not used
  hi_free = 0x3FF - lo_free;                             //       |
  _heapadd(d, lo_free);                                      // free low chunk
  _heapadd(b + 0x3FF, hi_free);                              // free high chunk

  n = fread(b, sizeof(char), 1024, fp);                  // read font file
  if (n != 1024) {
    return(NULL);
    fclose(fp);
  } 

  fclose(fp);
  return(b);
} 
