/* FRCT4CLN.C  -  Filled rect function, 4 color mode, no clipping
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * December 11, 2002
 */

#include <atari.h>
#include "atarigfx.h"


const unsigned char fill4cdata[5] = {0x55, 0xAA, 0xFF, 0x00, 0x00}; 



/* rectfill_4colmode_nc
 *
 * Draws a filled rectangle the bitmap using the passed in color register.
 * This function does no clipping, so you have to watch the parameters you
 * pass in.
 */
void __fastcall__ rectfill_4colmode_nc(BITMAP *bmp, unsigned int x, unsigned int y, unsigned w, unsigned h, unsigned char creg)
{
  unsigned char bit;                            // start pixel, end pixel
  unsigned char data;                           // screen data
  unsigned int index, si;                       // index to screen data
  unsigned int i, j;


  si = (bmp->bpl * y) + (x / 4);                    // start index

  for (i=0; i<h; ++i) {                             // height loop
    bit = (x % 4);                                  // pixel
    index = si;

    for(j=0; j<w; ++j) {                            // width loop
      if ((!bit) && ((j+4) < w)) {                  // fill whole byte? 
	data = fill4cdata[creg];
	bmp->dat[index] = data;

	j += 4;                                     // next byte
	bit = 0; 
	++index;
      }
      else { 
	data = bmp->dat[index];
	data &= plot4cmask[bit];                    // clear the pixel first

	if (creg != BAKCOL(bmp->mode)) {            // not background color?
	  data |= ((creg+1) << plot4cshift[bit]);   // creg 0->01, 1->10, 2->11
	}
	bmp->dat[index] = data;

	++bit;
	if (bit > 3) {
	  bit = 0;
	  ++index;
	}
      }
    }

    si += bmp->bpl;                                 // next screen line
  }

  return;
}
