/* MICLOAD.C  -  Micro Illustrator load function
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdio.h>
#include "atarigfx.h"



/* load_bitmap_mic
 *
 * Loads a Micro Illustrator compressed .pic file into the specified bitmap.
 * Returns 0 if file not found, 1 if data was loaded from the file.  Note
 * that this routine expects the bitmap height and width to be normal
 * (screen size) for ANTIC mode E.  This function does set the color
 * registers.  This function doesn't do clipping!
 *
 * Micro Illustrator format:
 *
 * Byte 0-6: unused?
 * Byte 7: vertical or horizontal compression? (1 = vert, 2 = horiz)
 * Byte 13-17: color registers (708-712)
 * Byte 18-26: unused?
 *
 * Data follows, encoded as <opcode><data>
 * opcode = 0       <hi byte><low byte><data byte>
 * opcode = <= 128  <data byte>
 * opcode = > 128   <(opcode - 128) data bytes>
 */
unsigned char __fastcall__ load_bitmap_mic(BITMAP *bmp, char *filename)
{
  FILE *pic;
  unsigned char *buf;
  unsigned char opcode;
  unsigned int i;
  unsigned int n;
  unsigned char hdr[27];


  //*** Open the file ***//
  pic = fopen(filename, "rb");
  if (!pic) return(0);

  //*** Read the header ***//
  fread(&hdr, sizeof(char), 27, pic);

  buf = bmp->dat;           // start of screen buffer
  hdr[25] = 0;              // vertical line counter
  hdr[26] = 0;              // horizontal line counter

  printf("file: %s\n", filename);
  printf("mode: %d colors: %x %x %x %x %x\n", hdr[7], hdr[13], hdr[14], hdr[15], hdr[16], hdr[17]);

  while(!feof(pic)) {
    fread(&opcode, sizeof(char), 1, pic);
    printf("op: %d", opcode);

    if (opcode <= 128) {
      if (opcode == 0) {
	fread(&hdr[0], sizeof(char), 2, pic);      // repetitions, hi/lo
	n = (hdr[0] * 256) + hdr[1];              // high*256+low 
	fread(&hdr[0], sizeof(char), 1, pic);      // data byte

	printf(" %d, %ld times\n", hdr[0], n);

	for(i=0; i<n; i++) {
	  buf[0] = hdr[0];

	  if (hdr[7] == 1) {
	    buf += 40;
	    hdr[26]++;
	    if (hdr[26] > 191) {
	      hdr[26] = 0;
	      hdr[25]++;
	      buf = (bmp->dat) + hdr[25];
	    }
	  }
	  else
	    buf++;
	}
     }
      else {
	fread(buf, sizeof(char), 1, pic);
	printf(" %d\n", buf[0]);
      }

      if (hdr[7] == 1) { 
	buf += 40;                                 // next line
	hdr[26]++;
      }
      else
	buf++; 
    }
    else if (opcode > 128) { 
      fread(&hdr[0], sizeof(char), 1, pic);
      printf(" %d, %d times", hdr[0], opcode-128); 

      for(i=0; i<(opcode-128); i++) {
	buf[0] = hdr[0];

	if (hdr[7] == 1) {
	  buf += 40;
	  hdr[26]++;
	}
	else
	  buf++;
      } 

      printf("\n");
    }

    if (hdr[26] > 191) {
      hdr[26] = 0; 
      hdr[25]++;
      buf = bmp->dat + hdr[25];
    }
  }

  fclose(pic);
  return(1);
} 
