/* PLOT4CLC.C  -  Plot function, 4 color mode, clipping
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * December 11, 2002
 */

#include <atari.h>
#include "atarigfx.h"



/* plot_4colmode
 *
 * Plots a point on the bitmap using the passed in color register.  This
 * function does clipping if the bitmap has clipping turned on.
 */
void __fastcall__ plot_4colmode(BITMAP *bmp, int x, int y, unsigned char creg)
{
  unsigned char bit;
  unsigned char data;
  unsigned int index;


  if ((x < 0) || (y < 0) || (x >= bmp->w) || (y >= bmp->h))
    return;

  bit  = (x % 4);
  index = (bmp->bpl * y) + (x / 4);

  data = bmp->dat[index];
  data &= plot4cmask[bit];                        // clear the pixel first

  if (creg != BAKCOL(bmp->mode)) {                // not background color?
    data |= ((creg+1) << plot4cshift[bit]);       // creg 0->01, 1->10, 2->11 
  }

  bmp->dat[index] = data;
  return;
}
