/* PMCREATE.C  -  Create a player/missile area
 *
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * Feb 7, 2003
 */

#include <atari.h>
#include <stdlib.h>
#include <stdio.h>
#include "atarigfx.h"


extern void __fastcall__ _heapadd(void *mem, size_t size);



/* create_pmarea
 *
 * Creates a player missile area, either 1k or 2k long depending on the
 * mode passed in.  It will be created on either a 2k or 1k boundary.  The
 * pm area is not activated by this function.
 *
 * Note: we also free the unused chunk at the beginning of the pmarea back
 * to the heap using hadd.  Don't try to use this bit of memory yourself
 * since it may be allocated for something else in your program.
 *
 * Use destroy_pmarea to free the pmarea memory.
 */
PMAREA __fastcall__ *create_pmarea(unsigned char mode)
{
  PMAREA *pmarea;
  char *d, *b;                          //           |
  unsigned int lo_free;                 //           |
  unsigned int hi_free;                 //           | 
  unsigned int bound;


  //*** Allocate a pmarea structure ***//
  pmarea = (PMAREA *) malloc(sizeof(PMAREA));
  if (!pmarea) return(NULL);

  pmarea->mode = mode;

  if (pmarea->mode == PM_2LINE)
    bound = 0x3FF;                      // 1k
  else
    bound = 0x800;                      // 2k

  //*** Allocate an aligned block of memory for the display list ***//
  d = (char *) malloc(bound + bound);                       // allocate mem
  if (!d) return(NULL);                                     // mem not avail

  b = (char *) ((((unsigned int) d) + bound) & ~bound);     // aligned pointer

  lo_free = b - d;                                          // memory not used
  hi_free = bound - lo_free;                                //       |
  _heapadd(d, lo_free);                                         // free low
  _heapadd(b + bound, hi_free);                                 // free high

  //*** Free unused portion back to the heap ***//
  if (pmarea->mode == PM_2LINE)
    _heapadd(b, 384);                       // 384 bytes free with double line
  else
    _heapadd(b, 768);                       // 768 bytes free with single line

  pmarea->dat = b;

  return(pmarea);
}
