/* TOUT2CAN.C  -  Textout 2 color mode, byte aligned, no clipping
 * 
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"



/* textout_2colmode_aligned_nc
 *
 * Writes text into bitmap without clipping.  Only use this function with
 * the 2 color modes where your x value is byte aligned (multiple of 8) since
 * it just writes straight bytes into the bitmap memory. Make sure you are not
 * clobbering memory beyond your bitmap.  Color of the text is controlled
 * through different means (color registers)  Font will be that which is
 * pointed to passed in value and may be the system font.
 */
void __fastcall__ textout_2colmode_aligned_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y)
{ 
  int tx, ty;
  char c;                       // character
  unsigned int i;               // string counter 
  unsigned char n;              // line counter
  unsigned char *line;          // character line pointer


  tx = x / 8;

  for (i=0; i < strlen(s); i++) {               // for each character in string
    c = atascii_to_internal(s[i]);              // internal representation
    line = font + (c * 8);                      // character line
    ty = y;                                     // start y position

    for(n=0; n<8; n++) {                        // for each line in character
      //*** NOTE: we could speed things up by doing most
      //*** of the next calculation outside the loop and
      //*** just adding the bitmap bytes per line each iteration.

      bmp->dat[(BYTES_PER_LINE(bmp) * ty) + tx] = *line;
      line++;                                   // next character line
      ty++;                                     // move one screen line down
    }

    tx++;                                       // next byte in bitmap
  }

  return;
}
