/* TOUT2CLN.C  -  Textout 2 color mode, no clipping
 * 
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"



/* textout_2colmode_nc
 * 
 * Writes text to the bitmap without clipping, so you will have to watch you
 * aren't clobbering memory beyond your bitmap.  Use this function with
 * the two color bitmap modes (Antic 9, B, F). Color is the color
 * register to use for plotting the text, usually 0 or background (4).
 *
 * Font is a pointer to a font in memory, and can be the system font ($E000).
 */
void __fastcall__ textout_2colmode_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg)
{
  unsigned int i;                       // string counter
  unsigned char n, m;                   // byte and bit counters
  int tx, ty, sx;                       // screen position
  char c;                               // character
  unsigned char *line;                  // character line pointer in font


  //*** Out of range of bitmap? ***//
  //if ((y < -7) || (y > bmp->h))
    //return;

  //*** Initialize ***//
  tx   = x;
  sx   = tx;

  for(i=0; i<strlen(s); i++) {
    c = atascii_to_internal(s[i]);                // convert to internal
    line = font + (c * 8);                        // offset to font data
    ty = y;

    for (n=0; n<8; n++) {                         // each line in character 
      for (m=128; m > 1; m = m / 2) {             // each bit in line 
	if (*line & m)
	  plot_2colmode(bmp, tx, ty, creg);
	tx++;
      }

      tx = sx;                                  // start of char on screen
      line++;                                   // next line of character
      ty++;                                     // next screen line down
    } 

    sx += 8;                                    // next char pos on screen
    tx = sx;
  }

  return;
}
