/* TOUTGTN.C  -  Textout gtia mode, no clipping
 * 
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * January 22, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"



/* textout_gtiamode_nc
 * 
 * Writes text to the bitmap without clipping.  Use this function with
 * the gtia bitmap modes (Antic F, Gr.9,10,11), and make sure you are not
 * clobbering memory beyond your bitmap.  Color is the color register (or
 * fixed color/luminance depending on mode) to use for plotting the text.
 *
 * Font is a pointer to a font in memory, and can be the system font.
 */
void __fastcall__ textout_gtiamode_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, char creg)
{
  unsigned int i;                       // string counter
  unsigned char n, m;                   // byte and bit counters
  int tx, ty, sx;                       // screen position
  char c;                               // character
  unsigned char *line;                  // character line pointer in font


  //*** Initialize ***//
  tx   = x;
  sx   = tx;

  for(i=0; i<strlen(s); i++) {
    c = atascii_to_internal(s[i]);                // convert to internal
    line = font + (c * 8);                        // offset to font data
    ty = y;

    for (n=0; n<8; n++) {                         // each line in character 
      for (m=128; m > 1; m = m / 2) {             // each bit in line 
	if (*line & m)
	  plot_gtiamode_nc(bmp, tx, ty, creg);
	tx++;
      }

      tx = sx;                                  // start of char on screen
      line++;                                   // next line of character
      ty++;                                     // next screen line down
    } 

    sx += 8;                                    // next char pos on screen
    tx = sx;
  }

  return;
}
