/* TOUTTXTC.C  -  Textout textmode with clipping
 * 
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include <stdlib.h>
#include <string.h>
#include "atarigfx.h"



/* textout_textmode
 *
 * Writes text into bitmap with clipping if turned on for the bitmap used.
 * Only use this function with the text modes. Color of the text is controlled
 * through different means (color registers and/or character set.)  Font will
 * be the current system font which is controlled through CHBAS (756).
 */
void __fastcall__ textout_textmode(BITMAP *bmp, const char *s, int x, int y)
{
  char *ts;                     // temporary string
  int sx;                       // start draw location
  int i;                        // counter
  int l;                        // string length


  //*** Check all outside bitmap area? ***//
  if ((y < 0) || (y > bmp->h))
    return;

  //*** Allocate temporary string storage ***//
  ts = (char *) malloc(sizeof(char) * (strlen(s) + 1));
  if (!ts) {
    return;
  }

  //*** Clipping? ***//
  if ((x+strlen(s)) > bmp->w) {
    l = 0;
    for (i=0; i<strlen(s); i++) {
      if (((x + i) >= 0) && ((x + i) < bmp->w)) {
	ts[l] = atascii_to_internal(s[i]);
	l++;
      } 
    }
  }
  else {
    strcpy(ts, s);
    str_to_internal(ts); 
  }

  //*** Draw onto bitmap ***//
  if (x < 0)
    sx = 0;
  else
    sx = x;

  memcpy(bmp->dat + ((bmp->w * y) + sx), ts, l);

  free(ts);
  return;
}
