/* TXTOUTNC.C  -  Generic textout function, clipping
 * 
 * Atari Graphics Library
 * Shawn Jefferson
 *
 * May 30, 2002
 */

#include <atari.h>
#include "atarigfx.h"



/* textout_nc
 *
 * Writes text onto a bitmap at the specified location using the specified
 * color register.  This function passes control to the specific textout
 * function for the bitmap mode.  This may mean that the color register
 * is not appropriate for the bitmap mode, the function does no conversion.
 * This function does clipping.
 */
void __fastcall__ textout_nc(BITMAP *bmp, FONT *font, const char *s, int x, int y, unsigned char creg)
{
  switch(bmp->mode) {
    case 0x02:                      // textmodes, ignore font and creg
    case 0x04:
    case 0x05:
    case 0x06:
    case 0x07: 
      textout_textmode_nc(bmp, s, x, y);
      break;

    case 0x09:                      // 2 color modes
    case 0x0B:
    case 0x0C:
    case 0x0F:
      textout_2colmode_nc(bmp, font, s, x, y, creg);
      break; 

    case 0x08:                      // 4 color modes
    case 0x0A:
    case 0x0D:
    case 0x0E:
      textout_4colmode_nc(bmp, font, s, x, y, creg);
      break;

    case 0x10:                      // gtia modes
    case 0x11:
    case 0x12:
      textout_gtiamode_nc(bmp, font, s, x, y, creg);
      break;
  }

  return;
}
